@@ -10,7 +10,7 @@ use File::Temp 'tempfile';
 use FindBin '$Bin';
 use Test::Builder;
 
-our $VERSION = '0.20';
+our $VERSION = '0.21';
 
 sub new
 {
@@ -164,11 +164,10 @@ sub test_if_modified_since
     $self->{no_warn} = 1;
     run_private ($self);
     $self->check_headers_private ($self);
-    my $headers = $run_options{headers};
-    $self->do_test ($headers->{status}, "Got a status header");
-    $self->do_test ($headers->{status} =~ /304/, "Got 304 status response");
+    $self->test_status (304);
     my $body = $run_options{body};
-    $self->do_test (! defined ($body) || length ($body) == 0, "No body returned with 304 response");
+    $self->do_test (! defined ($body) || length ($body) == 0,
+		    "No body returned with 304 response");
     $ENV{HTTP_IF_MODIFIED_SINCE} = $saved;
     # Restore our precious stuff.
     $self->{run_options} = $saved_run_options;
@@ -658,13 +657,28 @@ sub test_not_implemented
     run_private ($self);
     #print $options{output}, "\n";
     $self->check_headers_private ();
-    my $headers = $options{headers};
-    $self->{tb}->ok ($headers->{status}, "Got status header");
-    $self->{tb}->like ($headers->{status}, qr/501/, "Got not implemented status");
+    $self->test_status (501);
     $self->{no_check_content} = $saved_no_check_content;
     $self->clear_env ();
 }
 
+sub test_status
+{
+    my ($self, $status) = @_;
+    if ($status !~ /^[0-9]{3}$/) {
+	carp "$status is not a valid HTTP status, use a number like 301 or 503";
+	return;
+    }
+    my $headers = $self->{run_options}{headers};
+    if (! $headers) {
+	carp "no headers in this object; have you run a test yet?";
+	return;
+    }
+    $self->{tb}->ok ($headers->{status}, "Got status header");
+    $self->{tb}->like ($headers->{status}, qr/$status/, "Got $status status");
+} 
+
+
 sub test_method_not_allowed
 {
     my ($self, $bad_method) = @_;
@@ -682,11 +696,18 @@ sub test_method_not_allowed
     $tb->like ($headers->{status}, qr/405/, "Got method not allowed status");
     $self->clear_env ();
     my @allow = split /,\s*/, $headers->{allow};
+    my $saved_no_warn = $self->{no_warn};
+    $self->{no_warn} = 1;
     for my $ok_method (@allow) {
 	# Run the program with each of the headers we were told were
 	# allowed, and see whether the program executes correctly.
 	my %op2;
 	$op2{REQUEST_METHOD} = $ok_method;
+	if ($ok_method eq 'POST') {
+	    $op2{CONTENT_TYPE} = 'application/x-www-form-urlencoded';
+	    $op2{input} = 'a=b';
+#	    $op2{CONTENT_LENGTH} = length ($op2{input});
+	}
 	$self->{run_options} = \%op2;
 	run_private ($self);
 	$self->check_headers_private ();
@@ -697,14 +718,28 @@ sub test_method_not_allowed
 		 "Method $ok_method specified by Allow: header was allowed");
 	$self->clear_env ();
     }
+    $self->{no_warn} = $saved_no_warn;
     $self->{no_check_content} = $saved_no_check_content;
 }
 
+# Send bullshit queries expecting a 400 response.
+
+sub test_broken_queries
+{
+    my ($tester, $options, $queries) = @_;
+    for my $query (@$queries) {
+	$ENV{QUERY_STRING} = $query;
+	$tester->run ($options);
+	# test for 400 header
+	$tester->test_status (400);
+    }
+}
+
 # Clear all the environment variables we have set ourselves.
 
 sub clear_env
 {
-my ($self) = @_;
+    my ($self) = @_;
     for my $e (@{$self->{set_env}}) {
 #        print "Deleting environment variable $e\n";
         $ENV{$e} = undef;
